package com.softslate.commerce.administrator.payment;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * Processes the settings submitted from the PayPal UK settings screen. The
 * "paypalUKCertPath" is defined in this class as the form is submitted, with
 * the path to the cert in the "/WEB-INF/conf/keys" directory.
 * 
 * @author Ashish Jain
 * 
 */
public class SettingsFormPayPalUK extends BaseDynaForm {

	private static final long serialVersionUID = -7052890673228208303L;

	static Log log = LogFactory.getLog(SettingsFormPayPalUK.class);

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isDebugEnabled())
			log.debug("Starting validation.");

		initializeProperties(mapping, request);

		if (this.get("paypalUKCertPath") == null
				|| this.get("paypalUKCertPath").equals("")) {
			this.set("paypalUKCertPath", getServlet().getServletContext()
					.getRealPath("/WEB-INF/conf/keys"));
		}

		if (request.getParameter("paypalUKPaymentType") == null
				|| request.getParameter("paypalUKPaymentType").length() == 0) {
			String message = getMessage("payment", "paypalUK.paymentType", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		concatenateArrayAndSet((String[]) get("paypalUKAVSStArray"),
				"paypalUKAVSSt");
		concatenateArrayAndSet((String[]) get("paypalUKAVSZipArray"),
				"paypalUKAVSZip");
		concatenateArrayAndSet((String[]) get("paypalUKCVV2Array"),
				"paypalUKCVV2");

		if (request.getParameter("paypalUKHostAddress") == null
				|| request.getParameter("paypalUKHostAddress").length() == 0) {
			String message = getMessage("payment", "paypalUK.hostAddress", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		if (request.getParameter("paypalUKHostPort") == null
				|| request.getParameter("paypalUKHostPort").length() == 0) {
			String message = getMessage("payment", "paypalUK.hostPort", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		if (request.getParameter("paypalUKUser") == null
				|| request.getParameter("paypalUKUser").length() == 0) {
			String message = getMessage("payment", "paypalUK.user", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		if (request.getParameter("paypalUKPassword") == null
				|| request.getParameter("paypalUKPassword").length() == 0) {
			String message = getMessage("payment", "paypalUK.password", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		if (request.getParameter("paypalUKPartner") == null
				|| request.getParameter("paypalUKPartner").length() == 0) {
			String message = getMessage("payment", "paypalUK.partner", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		return getErrors();
	}

	protected String concatenateArray(String[] paypalUKArray) {
		String temp = null;

		if (null != paypalUKArray)
			for (int i = 0; i < paypalUKArray.length; i++)
				if (i != 0)
					temp = temp + "," + paypalUKArray[i];
				else
					temp = paypalUKArray[i];

		return temp;
	}

	protected void concatenateArrayAndSet(String[] paypalUKAVSStArray,
			String property) {
		String temp = this.concatenateArray(paypalUKAVSStArray);
		this.set(property, temp);
	}

}
