package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to "/administrator/CategoryAdd.do". Processes a request to add a
 * record to the system.
 * 
 * @author David Tobey
 */
public class CategoryAddAction extends CategoryEditAction {

	static Log log = LogFactory.getLog(CategoryAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Add the record
		// BaseAdministratorForm baseForm = (BaseAdministratorForm) form;
		CategoryAddEditForm baseForm = (CategoryAddEditForm) form;

		// Try uploading any files, these will throw exceptions if they do not
		// work therefore the category will not be added in that case.
		ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory()
				.createUtilsObject("actionUtilsImplementer");
		FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory()
				.createObject("fileStorageProcessorImplementer");

		// Load the imageSystemPath and imagePathURL settings; derive the
		// imageSystemPath from the URL path if blank.
		String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
		String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
		if (!imagePathURL.endsWith("/")) {
			imagePathURL = imagePathURL + "/";
		}
		if (imageSystemPath == null || imageSystemPath.length() == 0) {
			imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
					+ System.getProperty("file.separator");
		}
		if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
			imageSystemPath = imageSystemPath + System.getProperty("file.separator");
		}
		imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;
		String categoryOrContent = "Category";
		if (baseForm.getIsContentPage() != null && baseForm.getIsContentPage().equals("true")) {
			categoryOrContent = "Content";
		}

		if (baseForm.getSmallImageUpload() != null && baseForm.getSmallImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getSmallImageUpload().getFileName());
			if (validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + filename, "small")) {
				fsp.storeFile(baseForm.getSmallImageUpload().getInputStream(), imageSystemPath + filename,
						"small" + categoryOrContent);
				baseForm.setSmallImage(fsp.urlForFile(imagePathURL, filename, "small" + categoryOrContent));
			}
		}
		if (baseForm.getMediumImageUpload() != null && baseForm.getMediumImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getMediumImageUpload().getFileName());
			if (validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + filename, "medium")) {
				fsp.storeFile(baseForm.getMediumImageUpload().getInputStream(), imageSystemPath + filename,
						"medium" + categoryOrContent);
				baseForm.setMediumImage(fsp.urlForFile(imagePathURL, filename, "medium" + categoryOrContent));
			}
		}
		if (baseForm.getLargeImageUpload() != null && baseForm.getLargeImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getLargeImageUpload().getFileName());
			if (validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + filename, "large")) {
				fsp.storeFile(baseForm.getLargeImageUpload().getInputStream(), imageSystemPath + filename,
						"large" + categoryOrContent);
				baseForm.setLargeImage(fsp.urlForFile(imagePathURL, filename, "large" + categoryOrContent));
			}
		}

		// Handle generating the code from the name if it is empty
		if ((baseForm.getCode() == null || baseForm.getCode().equals(""))
				&& (baseForm.getName() != null && !baseForm.getName().equals(""))) {
			baseForm.setCode(actionUtils.generateCodeFromName(baseForm.getName()));
		}

		CategoryProcessor processor = (CategoryProcessor) baseForm.getBusinessObjectFactory()
				.createObject("categoryProcessorImplementer");
		Map results = processor.addCategory(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Reload category tree and builtin categories in the application
			// scope.
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeCategoryTree(baseForm.getBusinessObjectFactory());
			pi.initializeBuiltInCategories(baseForm.getBusinessObjectFactory());

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Category newRecord = (Category) results.get("newRecord");

				// entity.jsp wants the new record in the request scope.
				request.setAttribute("category", newRecord);

				// Set success message
				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.editNewRecord",
						"CategoryEditForm.do?categoryID=" + newRecord.getCategoryID()));
				request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);

				// Optionally update the Lucene index
				updateAdvancedSearchIndex(baseForm, newRecord.getCategoryID());
			}

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");
		} else if (results != null && results.get("existingRecord") != null) {

			// Set the error messages
			Category existingRecord = (Category) results.get("existingRecord");
			String existingCode = StringEscapeUtils.escapeXml11(existingRecord.getCode());
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.resultCode" + results.get("resultCode"), results.get("duplicateField"), existingCode));
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"CategoryEditForm.do?categoryID=" + existingRecord.getCategoryID(),
							results.get("duplicateField"), existingCode));
		}
		if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
			log.info("Validation error(s): " + baseForm.getErrors().toString());
		}
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}

	private boolean validateImageBeingOverwritten(CategoryAddEditForm baseForm, FileStorageProcessor fsp,
			String fullPathForFile, String fileType) {
		if (!baseForm.getOverwriteImageFiles() && fsp.doesFileExist(fullPathForFile, fileType)) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.imageWouldBeOverwritten", fileType));
			return false;
		}
		return true;
	}
}
