package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/CategoryEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>CategoryAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 * 
 */
public class CategoryEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CategoryEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		CategoryAddEditForm baseForm = (CategoryAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		CategoryProcessor processor = (CategoryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"categoryProcessorImplementer");
		Category category = null;
		if (baseForm.getCategoryID() != null) {
			category = processor.loadCategoryFromID(PropertyUtils.describe(form));
		}
		if (category == null && baseForm.getCode() != null) {
			category = processor.loadCategoryFromCode(PropertyUtils.describe(form));
		}
		BeanUtils.copyProperties(baseForm, category);

		// Load all categories for the parent category drop down.
		Collection categories = processor.loadAllCategories();
		baseForm.getRequest().setAttribute("categories", categories);

		// Load the parent if an id is present
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("categoryEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Categories", "Category Details" });
			if (category.getIsContentPage()) {
				bcs = Arrays.asList(new String[] { "Home", "Products",
						"Product Details", "Content Pages",
						"Content Page Details" });
			}
			request.setAttribute("breadcrumbs", bcs);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Categories",
					"Category Details" });
			if (category.getIsContentPage()) {
				bcs = Arrays.asList(new String[] { "Home", "Content Pages",
						"Content Page Details" });
			}
			request.setAttribute("breadcrumbs", bcs);
		}

		request.setAttribute("category", category);
		request.setAttribute("categoryAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
