package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Option;
import com.softslate.commerce.businessobjects.product.OptionProcessor;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to "/administrator/OptionAdd.do". Processes a request to add a
 * record to the system.
 * 
 * @author David Tobey, Tony Zakula
 * 
 */
public class OptionAddAction extends BaseAction {

	static Log log = LogFactory.getLog(OptionAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		OptionAddEditForm baseForm = (OptionAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Process any uploaded images
		ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory()
				.createUtilsObject("actionUtilsImplementer");
		FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory()
				.createObject("fileStorageProcessorImplementer");

		// Load the imageSystemPath and imagePathURL settings; derive the
		// imageSystemPath from the URL path if blank.
		String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
		String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
		if (!imagePathURL.endsWith("/")) {
			imagePathURL = imagePathURL + "/";
		}
		if (imageSystemPath == null || imageSystemPath.length() == 0) {
			imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
					+ System.getProperty("file.separator");
		}
		if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
			imageSystemPath = imageSystemPath + System.getProperty("file.separator");
		}
		imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;

		// Now process any uploaded images
		if (baseForm.getSmallImageUpload() != null && baseForm.getSmallImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getSmallImageUpload().getFileName());
			if (validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + filename, "smallOption")) {
				fsp.storeFile(baseForm.getSmallImageUpload().getInputStream(), imageSystemPath + filename,
						"smallOption");
				baseForm.setOptionSmallImage(fsp.urlForFile(imagePathURL, filename, "smallOption"));
			}
		}
		if (baseForm.getLargeImageUpload() != null && baseForm.getLargeImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getLargeImageUpload().getFileName());
			if (validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + filename, "largeOption")) {
				fsp.storeFile(baseForm.getLargeImageUpload().getInputStream(), imageSystemPath + filename,
						"largeOption");
				baseForm.setOptionLargeImage(fsp.urlForFile(imagePathURL, filename, "largeOption"));
			}
		}

		// Handle generating the code from the name if it is empty
		if ((baseForm.getOptionCode() == null || baseForm.getOptionCode().equals(""))
				&& (baseForm.getOptionName() != null && !baseForm.getOptionName().equals(""))) {
			baseForm.setOptionCode(actionUtils.generateCodeFromName(baseForm.getOptionName()));
		}

		OptionProcessor processor = (OptionProcessor) baseForm.getBusinessObjectFactory()
				.createObject("optionProcessorImplementer");
		Map results = processor.addOption(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Option newRecord = (Option) results.get("newRecord");

				// entity.jsp wants the new record in the request scope.
				request.setAttribute("option", newRecord);
				String link = "OptionEditForm.do?optionID=" + newRecord.getOptionID();
				if (PropertyUtils.getProperty(baseForm, "productID") != null) {
					link = link + "&productID=" + PropertyUtils.getProperty(baseForm, "productID");
				}
				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord", link));
				request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			}

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");

			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
				log.info("Validation error(s): " + baseForm.getErrors().toString());
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			Option existingRecord = (Option) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = StringEscapeUtils.escapeXml11(existingRecord.getOptionCode());
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			String link = "OptionEditForm.do?optionID=" + existingRecord.getOptionID();
			if (PropertyUtils.getProperty(baseForm, "productID") != null) {
				link = link + "&productID=" + PropertyUtils.getProperty(baseForm, "productID");
			}
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord", link, duplicateField, existingCode));
		} else if (results != null && results.get("resultCode") != null
				&& results.get("resultCode").equals("NoParentFound")) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode")));

		}
		if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
			log.info("Validation error(s): " + baseForm.getErrors().toString());
		}
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}

	private boolean validateImageBeingOverwritten(OptionAddEditForm baseForm, FileStorageProcessor fsp,
			String fullPathForFile, String fileType) {
		if (!baseForm.getOverwriteImageFiles() && fsp.doesFileExist(fullPathForFile, fileType)) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.imageWouldBeOverwritten", fileType));
			return false;
		}
		return true;
	}
}
