package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the API endpoint for listing ProductSettings. Used for requests to
 * "/api/productSetting/list".
 * <p>
 * The <code>execute</code> method of this class populates a <code>Collection</code> named <code>itemCollection</code>
 * in the request scope containing the items to be displayed. Next, it sets a request scope variable named
 * <code>editPermission</code> that determines if the current user has permission to edit the items. Finally, it creates
 * a <code>Collection</code> named <code>breadcrumbs</code> containing the links and labels of each breadcrumb for
 * display.
 * 
 * @author David Tobey
 * 
 */
public class ProductSettingAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		
		ProductSettingForm baseForm = (ProductSettingForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Handle the quick-add form
		ProductSettingAddEditForm addForm = (ProductSettingAddEditForm) request
				.getAttribute("productSettingAddEditForm");
		if (addForm == null
				|| (request.getAttribute("reset") != null && request
						.getAttribute("reset").equals("true"))) {
			addForm = new ProductSettingAddEditForm();
		}
		addForm.initializeProperties(mapping, request);
		request.setAttribute("productSettingAddEditForm", addForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		ProductSettingProcessor processor = bof.createObject(ProductSettingProcessor.class);
		Map productSettingsAndCount = processor.loadProductSettingsAndCount(PropertyUtils
				.describe(baseForm));
		request.setAttribute("productSettings", productSettingsAndCount.get("items"));
		request.setAttribute("itemCollection", productSettingsAndCount.get("items"));
		request.setAttribute("itemCount", productSettingsAndCount.get("itemCount"));
		request.setAttribute("assignedItems", productSettingsAndCount
				.get("assignedItems"));

		// Load the parent and grandparent if an id is present
		Product product = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getProductID());
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor gpProcessor =  bof
					.createObject(ProductProcessor.class);
			product = gpProcessor.loadProductFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("product", product);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("productSettingEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Products", "Product Details", "ProductSettings" });
			request.setAttribute("breadcrumbs", bcs);
			
			// Create childSubNavigation collection
			Collection childsn = Arrays.asList(new String[] {
					"Product Details", "ProductSettings", "Add ProductSetting" });
			request.setAttribute("childSubNavigation", childsn);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products", "Product Details", "ProductSettings" });
			request.setAttribute("breadcrumbs", bcs);
		}

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=productSettings.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
