package com.softslate.commerce.administrator.product;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.UUID;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Adds a setting to the system, but also uploads an image and resizes it for
 * display.
 * 
 * @author Tony Zakula
 */
public class ProductSettingAddImageAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingAddImageAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting Image Upload action.");

		ProductSettingAddEditFormImages baseForm = (ProductSettingAddEditFormImages) form;
		baseForm.initializeProperties(mapping, request);
		// Upload images and resize according to settings.
		if (baseForm.getAdditionalImageUpload() != null && baseForm.getAdditionalImageUpload().getFileSize() > 0) {

			ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
					"actionUtilsImplementer");
			FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory().createObject(
					"fileStorageProcessorImplementer");
			ProductSettingProcessor processor = (ProductSettingProcessor) baseForm.getBusinessObjectFactory()
					.createObject("productSettingProcessorImplementer");

			String smallImageSize = baseForm.getSettings().getValue("smallImageSize").toString();
			String mediumImageSize = baseForm.getSettings().getValue("mediumImageSize").toString();
			String largeImageSize = baseForm.getSettings().getValue("largeImageSize").toString();

			// Load the imageSystemPath and imagePathURL settings; derive the
			// imageSystemPath from the URL path if blank.
			String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
			String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
			if (!imagePathURL.endsWith("/")) {
				imagePathURL = imagePathURL + "/";
			}
			if (imageSystemPath == null || imageSystemPath.length() == 0) {
				imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
						+ System.getProperty("file.separator");
			}
			if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
				imageSystemPath = imageSystemPath + System.getProperty("file.separator");
			}
			imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;
			String filename = actionUtils.makeFilenameNice(baseForm.getAdditionalImageUpload().getFileName());

			// Validate files aren't being overwritten unless the user wants
			if (!validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + "LG-" + filename, "LARGE")
					|| !validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + "MD-" + filename, "MEDIUM")
					|| !validateImageBeingOverwritten(baseForm, fsp, imageSystemPath + "SM-" + filename, "SMALL")) {
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
					log.info("Validation error(s): " + baseForm.getErrors().toString());
				}
				saveErrors(baseForm.getRequest(), baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Save and insert images from the image that was uploaded, optionally scaling them.
			// Note that the global automaticallyResizeImages setting is not used here - it always creates small,
			// medium and large images from the source image that is uploaded.
			// Often the large image should not be resized but the system should just store the original, raw
			// image being uploaded. To make that happen simply set the largeImageSize setting to 0. Similarly,
			// to use the original, raw image for the medium or small image, set those sizes to 0 too.
			InputStream largeIS = null;
			InputStream smallIS = null;
			InputStream mediumIS = null;
			try {

				// Optionally scale, save and insert large image.
				int largeImageSizeInt = Integer.parseInt(largeImageSize);
				largeIS = baseForm.getAdditionalImageUpload().getInputStream();
				if (largeImageSizeInt > 0) {
					largeIS = resizeImage(baseForm.getAdditionalImageUpload().getInputStream(),
							largeImageSizeInt);
				}
				String largefilename = "LG-" + filename;
				fsp.storeFile(largeIS, imageSystemPath + largefilename, "largeAdditionalProduct");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName("LARGE");
				baseForm.setDescription(fsp.urlForFile(imagePathURL, largefilename, "largeAdditionalProduct"));
				processor.addProductSetting(PropertyUtils.describe(form));

				// Optionally scale, save and insert medium image.
				int mediumImageSizeInt = Integer.parseInt(mediumImageSize);
				mediumIS = baseForm.getAdditionalImageUpload().getInputStream();
				if (mediumImageSizeInt > 0) {
				mediumIS = resizeImage(baseForm.getAdditionalImageUpload().getInputStream(),
							mediumImageSizeInt);
				}
				String mediumfilename = "MD-" + filename;
				fsp.storeFile(mediumIS, imageSystemPath + mediumfilename, "mediumAdditionalProduct");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName("MEDIUM");
				baseForm.setDescription(fsp.urlForFile(imagePathURL, mediumfilename, "mediumAdditionalProduct"));
				processor.addProductSetting(PropertyUtils.describe(form));

				// Scale save and insert small image.
				int smallImageSizeInt = Integer.parseInt(smallImageSize);
				smallIS = baseForm.getAdditionalImageUpload().getInputStream();
				if (smallImageSizeInt > 0) {
					smallIS = resizeImage(baseForm.getAdditionalImageUpload().getInputStream(), smallImageSizeInt);
				}
				String smallfilename = "SM-" + filename;
				fsp.storeFile(smallIS, imageSystemPath + smallfilename, "smallAdditionalProduct");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName("SMALL");
				baseForm.setDescription(fsp.urlForFile(imagePathURL, smallfilename, "smallAdditionalProduct"));
				processor.addProductSetting(PropertyUtils.describe(form));
			} finally {
				if (largeIS != null)
					largeIS.close();
				if (mediumIS != null)
					mediumIS.close();
				if (smallIS != null)
					smallIS.close();
			}
		}

		// Set the success message
		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
		saveMessages(baseForm.getRequest(), baseForm.getMessages());

		// Reset the form
		Class classDefinition = baseForm.getClass();
		baseForm = (ProductSettingAddEditFormImages) classDefinition.newInstance();
		baseForm.initializeProperties(mapping, request);
		request.setAttribute("productSettingAddEditForm", baseForm);

		return mapping.findForward("success");
	}

	private boolean validateImageBeingOverwritten(ProductSettingAddEditFormImages baseForm, FileStorageProcessor fsp,
			String fullPathForFile, String fileType) {
		if (!baseForm.getOverwriteImageFiles() && fsp.doesFileExist(fullPathForFile, fileType)) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.imageWouldBeOverwritten", fileType));
			return false;
		}
		return true;
	}

	// Resize keeping the same aspect ratio. If the height is more than the width, set the height to the
	// incoming imageSize and adjust the width proportionally. If it's wider than it is tall, set the
	// width to the incoming setting and adjust the height proportionally.
	private InputStream resizeImage(InputStream inputStream, int imageSize) throws IOException {
		BufferedImage srcImage = ImageIO.read(inputStream);
		int srcWidth = srcImage.getWidth();
		int srcHeight = srcImage.getHeight();
		int newWidth = srcWidth;
		int newHeight = srcHeight;
		if (srcHeight > srcWidth) {
			newHeight = imageSize;
			double ratio = (double) imageSize / srcHeight;
			double unrounded = srcWidth * ratio;
			newWidth = (int) Math.round(unrounded);
		} else {
			newWidth = imageSize;
			double ratio = (double) imageSize / srcWidth;
			double unrounded = srcHeight * ratio;
			newHeight = (int) Math.round(unrounded);
		}
		BufferedImage scaledImage = new BufferedImage(newWidth, newHeight, srcImage.getType());
		Graphics2D g = scaledImage.createGraphics();
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
		g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
		g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
		g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
		g.drawImage(srcImage, 0, 0, newWidth, newHeight, 0, 0, srcImage.getWidth(), srcImage.getHeight(), null);
		g.dispose();
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		ImageIO.write(scaledImage, "jpg", os);
		byte[] scaledImageContents = os.toByteArray();
		InputStream is = new ByteArrayInputStream(scaledImageContents);
		os.close();
		return is;
	}

}
