package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to "/administrator/ProductSettingEditForm.do". Loads in the current
 * values of the record being edited and populates a <code>ProductSettingAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class ProductSettingEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ProductSettingAddEditForm baseForm = (ProductSettingAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		ProductSettingProcessor processor = (ProductSettingProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"productSettingProcessorImplementer");
		ProductSetting productSetting = null;
		if (baseForm.getProductSettingID() != null) {
			productSetting = processor.loadProductSettingFromID(PropertyUtils.describe(form));
		}
		if (productSetting == null && baseForm.getCode() != null) {
			productSetting = processor.loadProductSettingFromCode(PropertyUtils.describe(form));
		}
		BeanUtils.copyProperties(baseForm, productSetting);

		// Load the parent and grandparent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Product product = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getProductID());
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor gpProcessor =  bof
					.createObject(ProductProcessor.class);
			product = gpProcessor.loadProductFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("product", product);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("productSettingEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Product Settings", "Product Setting Details", "Product Settings",
					"Product Setting Details" });
			request.setAttribute("breadcrumbs", bcs);

			// Create childSubNavigation collection
			Collection childsn = Arrays.asList(new String[] {
					"Product Setting Details", "Product Settings", "Add Product Setting", "Product Setting Details" });
			request.setAttribute("childSubNavigation", childsn);

		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Product Settings", "Product Setting Details",
					"Product Settings", "Product Setting Details" });
			request.setAttribute("breadcrumbs", bcs);
		}

		request.setAttribute("productSetting", productSetting);
		request.setAttribute("productSettingAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
