package com.softslate.commerce.administrator.product;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class for builing sku records with the SKU builder. Used for
 * requests to "/administrator/SKUBuilder.do".
 * 
 * @author David Tobey
 */
public class SKUBuilderAction extends BaseAction {

	static Log log = LogFactory.getLog(SKUBuilderAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Build the skus according to
		ActionMessages messages = new ActionMessages();
		ActionMessages errors = new ActionErrors();
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		SKUProcessor processor =  bof
				.createObject(SKUProcessor.class);

		Map results = null;

		// Process the request and display messages
		if (request.getParameter("skusToBuild") == null) {
			return mapping.findForward("success");

		} else if (request.getParameter("skusToBuild").equals("productSkus")
				&& request.getParameter("productID") != null) {
			String[] includedAttributeIdsParam = request.getParameterValues("includedAttributeIds");
			List<Integer> includedAttributeIds = new ArrayList();
			if (includedAttributeIdsParam != null) {
				for (int i = 0; i < includedAttributeIdsParam.length; i++) {
					try {
						includedAttributeIds.add(Integer.parseInt(includedAttributeIdsParam[i]));
					} catch (NumberFormatException nfe) {
						log.info("Invalid attributeID: " + includedAttributeIdsParam[i]);
					}
				}
			}
			String[] includedOptionIdsParam = request.getParameterValues("includedOptionIds");
			List<Integer> includedOptionIds = new ArrayList();
			if (includedOptionIdsParam != null) {
				for (int i = 0; i < includedOptionIdsParam.length; i++) {
					try {
						includedOptionIds.add(Integer.parseInt(includedOptionIdsParam[i]));
					} catch (NumberFormatException nfe) {
						log.info("Invalid optionID: " + includedOptionIdsParam[i]);
					}
				}
			}
			results = processor.buildProductSKUs(Integer.parseInt(request.getParameter("productID")),
					includedAttributeIds, includedOptionIds);
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"messages.productSKUsBuilt", results.get("insertCount"),
					results.get("existsCount")));

		} else if (request.getParameter("skusToBuild").equals("allProductSKUs")) {
			results = processor.buildAllProductSKUs();
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"messages.allProductSKUsBuilt", results.get("insertCount"),
					results.get("existsCount")));

		} else if (request.getParameter("skusToBuild").equals(
				"allAttributeSKUs")) {
			results = processor.buildAllAttributeSKUs();
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"messages.allAttributeSKUsBuilt", results
							.get("insertCount"), results.get("existsCount")));
		}
		saveMessages(request, messages);

		// Show an error if there are too many attribute/option combinations
		if (results != null && results.get("numberOfAttributeOptionCombinations") != null) {
			Integer numberOfAttributeOptionCombinations = (Integer) results.get("numberOfAttributeOptionCombinations");
			int maxRecordsProcessed = Integer.parseInt(bof.getSettings().getValue("maxRecordsProcessed"));
			if (numberOfAttributeOptionCombinations > maxRecordsProcessed) {
				errors.add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.skuBuilderTooManySKUs", numberOfAttributeOptionCombinations,
								maxRecordsProcessed));
				saveErrors(request, (ActionMessages) errors);
			}
		}

		// Show an error if any of the sku numbers conflicted with existing sku
		// numbers
		if (results != null && results.get("skuNumberConflicts") != null) {
			Collection skuNumberConflicts = (Collection) results.get("skuNumberConflicts");
			if (skuNumberConflicts.size() > 0) {
				StringBuffer sb = new StringBuffer("");
				Iterator i = skuNumberConflicts.iterator();
				while (i.hasNext()) {
					String skuNumber = (String) i.next();
					if (sb.length() != 0) {
						sb.append(", ");
					}
					sb.append(skuNumber);
				}
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"errors.skuNumberConflicts", sb.toString()));
				saveErrors(request, (ActionMessages) errors);
			}
		}

		return mapping.findForward("success");
	}
}
