package com.softslate.commerce.administrator.product;

import java.util.ArrayList;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/SKUEditDelete.do". Processes a request to delete and edit
 * multiple records in the system. These requests are generated when the
 * adminstrator is deleting or editing multiple records simultaneously while in
 * edit mode.
 * 
 * @author David Tobey
 */
public class SKUEditDeleteAction extends BaseAction {

    static Log log = LogFactory.getLog(SKUEditDeleteAction.class);

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        if (log.isDebugEnabled())
            log.debug("Starting action.");
        SKUForm baseForm = (SKUForm) form;

        // Prepare parameters for the edit/delete processing, and process
        BusinessObjectFactory bof = (BusinessObjectFactory) request
                .getAttribute("businessObjectFactory");
        SKUProcessor processor =  bof
                .createObject(SKUProcessor.class);
        Map parameters = PropertyUtils.describe(form);
        ArrayList newDisplayFields = new ArrayList();
        for (int j = 0; j < baseForm.getDisplayFields().length; j++) {
            String displayField = baseForm.getDisplayFields()[j];
            // Do not update the product, or attributes and options
            if (!displayField.equals("productID")
            		&& !displayField.equals("attributeOption1")
            		&& !displayField.equals("attributeOption2")
            		&& !displayField.equals("attributeOption3")
            		&& !displayField.equals("attributeOption4")
            		&& !displayField.equals("attributeOption5")) {
                newDisplayFields.add(displayField);
                parameters.put(displayField, PropertyUtils.getProperty(
                        baseForm, displayField));
            }
        }
        parameters
                .put("displayFields", newDisplayFields.toArray(new String[1]));
        parameters.put(baseForm.getIdField(), PropertyUtils.getProperty(
                baseForm, baseForm.getIdField()));
        parameters.put("deleteID", baseForm.getDeleteID());
        processor.editDeleteSKUs(parameters);

		// Reload the store discounts in the application scope since the sku
		// associations may have changed
		ProductInitializer pi = new ProductInitializer();
		pi.setServlet(servlet);
		pi.initializeStoreDiscounts(bof);

        // Set the success message
        ActionMessages messages = new ActionMessages();
        messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                "messages.recordsSaved"));
        saveMessages(request, messages);

        return mapping.findForward("success");
    }
}
