package com.softslate.commerce.administrator.shipping;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.administrator.core.BaseAdministratorForm;

/**
 * Struts form class corresponding to requests to
 * "/administrator/ShippingMethodAddForm.do",
 * "/administrator/ShippingMethodEditForm.do",
 * "/administrator/ShippingMethodAdd.do", and
 * "/administrator/ShippingMethodEdit.do". Provides properties corresponding to
 * the fields for a single shipping method record.
 * 
 * @author David Tobey
 * 
 */
public class ShippingMethodAddEditForm extends BaseAdministratorForm {
	private static final long serialVersionUID = 4049353123816354608L;

	static Log log = LogFactory.getLog(ShippingMethodAddEditForm.class);

	private String shippingMethodID = null;

	private String code = null;

	private String name = null;

	private String minimum = "0.00";

	private String base = "0.00";

	private String methodType = null;

	private String isActive = "true";

	private String numberDaysToShip = null;

	private String methodOrder = null;

	public String getShippingMethodID() {
		return shippingMethodID;
	}

	public void setShippingMethodID(String shippingMethodID) {
		this.shippingMethodID = shippingMethodID;
	}

	public String getBase() {
		return base;
	}

	public void setBase(String base) {
		this.base = base;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getIsActive() {
		return isActive;
	}

	public void setIsActive(String isActive) {
		this.isActive = isActive;
	}

	public String getMethodType() {
		return methodType;
	}

	public void setMethodType(String methodType) {
		this.methodType = methodType;
	}

	public String getMinimum() {
		return minimum;
	}

	public void setMinimum(String minimum) {
		this.minimum = minimum;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getNumberDaysToShip() {
		return numberDaysToShip;
	}

	public void setNumberDaysToShip(String numberDaysToShip) {
		this.numberDaysToShip = numberDaysToShip;
	}

	public String getMethodOrder() {
		return methodOrder;
	}

	public void setMethodOrder(String methodOrder) {
		this.methodOrder = methodOrder;
	}

	private String isPickUp = "false";

	public String getIsPickUp() {
		return isPickUp;
	}

	public void setIsPickUp(String isPickUp) {
		this.isPickUp = isPickUp;
	}

	private String organization = null;

	public String getOrganization() {
		return (this.organization);
	}

	public void setOrganization(String organization) {
		this.organization = organization;
	}

	private String address1 = null;

	public String getAddress1() {
		return (this.address1);
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	private String address2 = null;

	public String getAddress2() {
		return (this.address2);
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	private String city = null;

	public String getCity() {
		return (this.city);
	}

	public void setCity(String city) {
		this.city = city;
	}

	private String state = null;

	public String getState() {
		return (this.state);
	}

	public void setState(String state) {
		this.state = state;
	}

	private String country = null;

	public String getCountry() {
		return (this.country);
	}

	public void setCountry(String country) {
		this.country = country;
	}

	private String postalCode = null;

	public String getPostalCode() {
		return (this.postalCode);
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isDebugEnabled())
			log.debug("Starting validation.");
		initializeProperties(mapping, request);
		if (!addingNewRecord() && StringUtils.isEmpty(getShippingMethodID()) && StringUtils.isEmpty(getCode())) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "shippingMethodID or code"));
		}
		if (getCode() != null && !getCode().equals("")
				&& containsUnsafeCharacter(getCode(), request) != null) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.unsafeCharacter", "The code "
							+ StringEscapeUtils.escapeXml11(getCode()), containsUnsafeCharacter(getCode(),
							request)));
		}
		if (addingAndFieldIsBlank(getName()) || editingAndFieldIsBeingEmptiedOut(getName())) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Name"));
		}
		if (addingAndFieldIsBlank(getMethodType()) || editingAndFieldIsBeingEmptiedOut(getMethodType())) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Method type"));
		}
		return getErrors();
	}

}
