package com.softslate.commerce.administrator.shipping;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.shipping.ShippingMethod;
import com.softslate.commerce.businessobjects.shipping.ShippingMethodProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/ShippingMethodEdit.do". Processes a request to edit a record within
 * the system.
 * 
 * @author David Tobey
 * 
 */
public class ShippingMethodEditAction extends BaseAction {

	static Log log = LogFactory.getLog(ShippingMethodEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		ShippingMethodAddEditForm baseForm = (ShippingMethodAddEditForm) form;

		// Handle generating the code from the name if it is empty
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		if ((baseForm.getCode() == null || baseForm.getCode().equals(""))
				&& (baseForm.getName() != null && !baseForm.getName()
						.equals(""))) {
			baseForm.setCode(actionUtils.generateCodeFromName(baseForm
					.getName()));
		}

		// Edit the record
		ShippingMethodProcessor processor = (ShippingMethodProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"shippingMethodProcessorImplementer");
		Map results = processor.editShippingMethod(baseForm.mapOfIncomingParams());

		// Successful update
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// entity.jsp wants the updated record in the request scope
			request.setAttribute("shippingMethod", results.get("updatedRecord"));

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordSaved"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			ShippingMethod existingRecord = (ShippingMethod) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = existingRecord.getCode();
			existingCode = StringEscapeUtils.escapeXml11(existingCode);
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			String link = "ShippingMethodEditForm.do?shippingMethodID=" + existingRecord.getShippingMethodID();
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord", link, duplicateField, existingCode));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		}

		return mapping.findForward("success");
	}
}
