package com.softslate.commerce.businessobjects.installer;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.core.Setting;
import com.softslate.commerce.businessobjects.core.SettingBean;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.SettingsBean;
import com.softslate.commerce.daos.core.BaseDAOFactory;
import com.softslate.commerce.daos.core.SettingsDAOHibernate;
import com.softslate.commerce.daos.installer.InstallerDAOBasic;

/**
 * Processes various requests from the installer tool to initialize the
 * application.
 * 
 * @author David Tobey
 * 
 */
public class InstallerProcessor extends BaseBusinessProcessor {

	static Log log = LogFactory.getLog(InstallerProcessor.class);

	private boolean insertTestData = false;

	public boolean getInsertTestData() {
		return (this.insertTestData);
	}

	public void setInsertTestData(boolean insertTestData) {
		this.insertTestData = insertTestData;
	}

	public void processInitialization(Map parameters) throws Exception {
		if (parameters != null) {
			BeanUtils.copyProperties(this, parameters);
		}
		if (log.isInfoEnabled())
			log.info("About to process the database initialization.");
		InstallerDAOBasic installerDAO = new InstallerDAOBasic();
		installerDAO.setDaoFactory(getDaoFactory());
		installerDAO.processInitialization(BaseDAOFactory.moduleListString,
				(String) parameters.get("pathToResources"),
				getInsertTestData());

		return;
	}

	public void processStoreSettings(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("About to process the store settings.");
		Settings settings = new SettingsBean();
		Setting setting = new SettingBean();
		SettingsDAOHibernate settingsDAO = new SettingsDAOHibernate();
		settingsDAO.setDaoFactory(getDaoFactory());
		settingsDAO.updateSettings(parameters, settings, setting);
		return;
	}

	public void processSecuritySettings(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("About to process the security settings.");
		Settings settings = new SettingsBean();
		Setting setting = new SettingBean();
		SettingsDAOHibernate settingsDAO = new SettingsDAOHibernate();
		settingsDAO.setDaoFactory(getDaoFactory());
		settingsDAO.updateSettings(parameters, settings, setting);
		if (log.isDebugEnabled())
			log.debug("About to save the administrator password.");
		InstallerDAOBasic installerDAO = new InstallerDAOBasic();
		installerDAO.setDaoFactory(getDaoFactory());
		installerDAO.processAdministratorInstall(parameters);
		return;
	}
}
