package com.softslate.commerce.businessobjects.order;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ecwid.mailchimp.MailChimpClient;
import com.ecwid.mailchimp.MailChimpException;
import com.ecwid.mailchimp.MailChimpObject;
import com.ecwid.mailchimp.method.v2_0.lists.Email;
import com.ecwid.mailchimp.method.v2_0.lists.SubscribeMethod;
import com.softslate.commerce.customer.core.FormUtils;
import com.softslate.emailer.businessobjects.core.BasicEmailerProcessor;

public class MailChimpProcessor extends BasicEmailerProcessor {

	static Log log = LogFactory.getLog(MailChimpProcessor.class);

	public Map register(Map parameters) throws Exception {
		return null;
	}

	public Map createClient(Map parameters) throws Exception {
		return null;
	}

	public Map updateAccessAndBilling(Map parameters) throws Exception {
		return null;
	}

	public Map deleteClient(Map parameters) throws Exception {
		return null;
	}

	public Map processContactForm(Map parameters) throws Exception {
		// Grab contact form settings for the behavior and the list name
		String behavior = (String) getSettings().getValue("mailChimpContactForm");
		String listID = (String) getSettings().getValue("mailChimpContactFormListID");

		// Process the integration
		return processIntegration(behavior, listID, parameters);
	}

	public Map processCustomerRegistration(Map parameters) throws Exception {

		// Grab customer registration settings for the behavior and the list
		// name
		String behavior = (String) getSettings().getValue("mailChimpCustomerRegistration");
		String listID = (String) getSettings().getValue("mailChimpCustomerRegistrationListID");

		// Process the integration
		return processIntegration(behavior, listID, parameters);
	}

	public Map processCheckoutAddresses(Map parameters) throws Exception {

		// Grab order placement settings for the behavior and the list name
		String behavior = (String) getSettings().getValue("mailChimpCheckout");
		String listID = (String) getSettings().getValue("mailChimpCheckoutListID");

		// Process the integration
		return processIntegration(behavior, listID, parameters);
	}

	public Map processIntegration(String behavior, String listID, Map parameters) throws Exception {

		// Set some defaults
		Map result = new HashMap();
		result.put("Code", "0");
		result.put("Message", "Success");
		if (log.isDebugEnabled())
			log.debug("parameters: " + parameters);

		// Check to see if we should add this contact
		String emailerOptInApproval = (String) parameters.get("emailerOptInApproval");
		boolean optedIn = emailerOptInApproval != null && emailerOptInApproval.equals("Yes");
		String email = (String) parameters.get("Email");
		FormUtils formUtils = (FormUtils) getBusinessObjectFactory().createUtilsObject("formUtilsImplementer");
		boolean validEmail = email != null && formUtils.isValidEmail(email);
		if (behavior.equals("dontSubscribe") || !validEmail || !(behavior.equals("subscribeAll") || optedIn)) {
			if (log.isInfoEnabled()) {
				log.info("Not adding contact: email=" + validEmail + " behavior=" + behavior + " optedIn=" + optedIn);
			}
			return null;
		}
		if (behavior.equals("subscribeAll") || (behavior.equals("subscribeOptIns") && optedIn)) {

			String mailChimpAPIKey = (String) getSettings().getValue("mailChimpAPIKey");
			String firstName = (String) parameters.get("First Name");
			String lastName = (String) parameters.get("Last Name");

			// Subscribe a person
			MailChimpClient mailChimpClient = new MailChimpClient();
			SubscribeMethod subscribeMethod = new SubscribeMethod();
			subscribeMethod.apikey = mailChimpAPIKey;
			subscribeMethod.id = listID;
			subscribeMethod.email = new Email();
			subscribeMethod.email.email = email;
			subscribeMethod.double_optin = false;
			if (firstName != null && lastName != null) {
				subscribeMethod.merge_vars = new MergeVars(email, firstName, lastName);
			}
			// Update existing will not throw an error if they are subscribed
			subscribeMethod.update_existing = true;

			try {
				mailChimpClient.execute(subscribeMethod);
				log.info("Subscribed " + email + " to MailChimp list");
			} catch (MailChimpException e) {
				log.warn("Exception in adding to list - " + e.message);
				result.put("Code", Integer.toString(e.code));
				result.put("Message", e.message);
			} finally {
				mailChimpClient.close();
			}
		}
		return result;
	}

	public static class MergeVars extends MailChimpObject {

		private static final long serialVersionUID = 1L;

		@Field
		public String EMAIL, FNAME, LNAME;

		public MergeVars() {
		}

		public MergeVars(String email, String fname, String lname) {
			this.EMAIL = email;
			this.FNAME = fname;
			this.LNAME = lname;
		}
	}

}
