package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with order item attributes.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting order item attributes.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderItemAttributeProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}<!--
 * --> finds the name of the Java class to instantiate from the
 * "orderItemAttributeProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderItemAttributeProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderItemAttributeProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface OrderItemAttributeProcessor extends BusinessProcessor {

	abstract Map loadOrderItemAttributesAndCount(Map parameters)
			throws Exception;

	abstract Map addOrderItemAttribute(Map parameters) throws Exception;

	abstract OrderItemAttribute loadOrderItemAttributeFromID(Map parameters)
			throws Exception;

	abstract Map editOrderItemAttribute(Map parameters) throws Exception;

	abstract Map editDeleteOrderItemAttributes(Map parameters)
			throws Exception;

	abstract void deleteOrderItemAttribute(Map parameters) throws Exception;
}
