package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Currency;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a product within the system. <code>ProductBean</code>
 * is the default implementation of the <code>Product</code> interface for the
 * application.
 * <p>
 * The database stores products in the <code>sscProduct</code> table, and this
 * interface can be used to represent a single row of that table. The Hibernate
 * configuration file <code>ProductBean.hbm.xml</code> maps that table's
 * columns to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudProduct")
public class ProductBean extends BaseBusinessObject implements Product,
		Serializable {

	private static final long serialVersionUID = 6016213050418840957L;

	static Log log = LogFactory.getLog(ProductBean.class);

	private int productID = 0;

	private String code = null;

	private String seoCode = null;

	private String name = null;

	private String keywords = null;

	private String shortDescription = null;

	private String description = null;

	private boolean isActive = true;

	private boolean isTaxed = true;

	private Double unitCost = null;

	private Double unitPrice = null;

	private Double altPrice = null;

	private Double weight = null;

	private Integer primaryCategoryID = null;

	private String header = null;

	private String footer = null;

	private String smallImage;

	private String mediumImage;

	private String largeImage;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private String extra4 = null;

	private String extra5 = null;

	private String created = null;

	private int productOrder = 0;

	private Integer manufacturerID = null;

	public Integer getManufacturerID() {
		return manufacturerID;
	}

	public void setManufacturerID(Integer manufacturerID) {
		this.manufacturerID = manufacturerID;
	}

	public int getProductID() {
		return (this.productID);
	}

	public void setProductID(int productID) {
		this.productID = productID;
	}

	public String getCode() {
		return (this.code);
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getName() {
		return (this.name);
	}

	public void setName(String name) {
		this.name = name;
	}

	public boolean getIsActive() {
		return (this.isActive);
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getExtra4() {
		return extra4;
	}

	public void setExtra4(String extra4) {
		this.extra4 = extra4;
	}

	public String getExtra5() {
		return extra5;
	}

	public void setExtra5(String extra5) {
		this.extra5 = extra5;
	}

	public String getKeywords() {
		return keywords;
	}

	public void setKeywords(String keywords) {
		this.keywords = keywords;
	}

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getShortDescription() {
		return shortDescription;
	}

	public void setShortDescription(String shortDescription) {
		this.shortDescription = shortDescription;
	}

	public String getDescription() {
		return (this.description);
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Double getUnitCost() {
		return (this.unitCost);
	}

	public void setUnitCost(Double unitCost) {
		this.unitCost = unitCost;
	}

	public Double getUnitPrice() {
		return (this.unitPrice);
	}

	public void setUnitPrice(Double unitPrice) {
		this.unitPrice = unitPrice;
	}

	public Double getAltPrice() {
		return (this.altPrice);
	}

	public void setAltPrice(Double altPrice) {
		this.altPrice = altPrice;
	}

	public Double getWeight() {
		return (this.weight);
	}

	public void setWeight(Double weight) {
		this.weight = weight;
	}

	public String getHeader() {
		return (this.header);
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public String getFooter() {
		return (this.footer);
	}

	public void setFooter(String footer) {
		this.footer = footer;
	}

	public boolean getIsTaxed() {
		return (this.isTaxed);
	}

	public void setIsTaxed(boolean isTaxed) {
		this.isTaxed = isTaxed;
	}

	public String getSmallImage() {
		return (this.smallImage);
	}

	public void setSmallImage(String smallImage) {
		this.smallImage = smallImage;
	}

	public String getMediumImage() {
		return (this.mediumImage);
	}

	public void setMediumImage(String mediumImage) {
		this.mediumImage = mediumImage;
	}

	public String getLargeImage() {
		return (this.largeImage);
	}

	public void setLargeImage(String largeImage) {
		this.largeImage = largeImage;
	}

	public Integer getPrimaryCategoryID() {
		return primaryCategoryID;
	}

	public void setPrimaryCategoryID(Integer primaryCategoryID) {
		this.primaryCategoryID = primaryCategoryID;
	}

	public int getProductOrder() {
		return productOrder;
	}

	public void setProductOrder(int productOrder) {
		this.productOrder = productOrder;
	}

	public String getSeoCode() {
		return seoCode;
	}

	public void setSeoCode(String seoCode) {
		this.seoCode = seoCode;
	}

	public String getCreated() {
		return created;
	}

	public void setCreated(String created) {
		this.created = created;
	}

	private String formattedCost;

	public String getFormattedCost() {
		return (this.formattedCost);
	}

	public void setFormattedCost(String formattedCost) {
		this.formattedCost = formattedCost;
	}

	private String formattedPrice;

	public String getFormattedPrice() {
		return (this.formattedPrice);
	}

	public void setFormattedPrice(String formattedPrice) {
		this.formattedPrice = formattedPrice;
	}

	private String formattedAltPrice;

	public String getFormattedAltPrice() {
		return (this.formattedAltPrice);
	}

	public void setFormattedAltPrice(String formattedAltPrice) {
		this.formattedAltPrice = formattedAltPrice;
	}

	private Collection attributesAndOptions = null;

	public Collection getAttributesAndOptions() {
		return (this.attributesAndOptions);
	}

	public void setAttributesAndOptions(Collection attributesAndOptions) {
		this.attributesAndOptions = attributesAndOptions;
	}

	private Map productSettings = null;

	public Map getProductSettings() {
		return productSettings;
	}

	public void setProductSettings(Map productSettings) {
		this.productSettings = productSettings;
	}

	private Collection productCategories = null;

	public Collection getProductCategories() {
		return (this.productCategories);
	}

	public void setProductCategories(Collection productCategories) {
		this.productCategories = productCategories;
	}

	private Category primaryCategory = null;

	public Category getPrimaryCategory() {
		return (this.primaryCategory);
	}

	public void setPrimaryCategory(Category primaryCategory) {
		this.primaryCategory = primaryCategory;
	}

	private Manufacturer manufacturer = null;

	public Manufacturer getManufacturer() {
		return manufacturer;
	}

	public void setManufacturer(Manufacturer manufacturer) {
		this.manufacturer = manufacturer;
	}

	private Collection productAttributes = null;

	public Collection getProductAttributes() {
		return (this.productAttributes);
	}

	public void setProductAttributes(Collection productAttributes) {
		this.productAttributes = productAttributes;
	}

	private Collection skus = null;

	public Collection getSkus() {
		return (this.skus);
	}

	public void setSkus(Collection skus) {
		this.skus = skus;
	}

	private Map skuMap = null;

	public Map getSkuMap() {
		if (skuMap != null) {
			return skuMap;
		}
		skuMap = buildSkuMap(getSkus());
		return (skuMap);
	}

	private Collection relatedProducts = null;

	public Collection getRelatedProducts() {
		return (this.relatedProducts);
	}

	public void setRelatedProducts(Collection relatedProducts) {
		this.relatedProducts = relatedProducts;
	}

	private Collection relatedProductsAdmin = null;

	public Collection getRelatedProductsAdmin() {
		return (this.relatedProductsAdmin);
	}

	public void setRelatedProductsAdmin(Collection relatedProductsAdmin) {
		this.relatedProductsAdmin = relatedProductsAdmin;
	}
	
	private Collection customerWishListProducts = null;

	public Collection getCustomerWishListProducts() {
		return (this.customerWishListProducts);
	}

	public void setCustomerWishListProducts(Collection customerWishListProducts) {
		this.customerWishListProducts = customerWishListProducts;
	}
	
	private Collection productReviews = null;

	public Collection getProductReviews() {
		return (this.productReviews);
	}

	public void setProductReviews(Collection productReviews) {
		this.productReviews = productReviews;
	}

	
	public void addFormatting(Locale locale, String currencyCode) {
		if (currencyCode == null) {
			currencyCode = "USD";
		}
		NumberFormat nf = NumberFormat.getCurrencyInstance(locale);
		Currency currency = Currency.getInstance(currencyCode);
		nf.setCurrency(currency);
		if (getUnitCost() != null) {
			setFormattedCost(nf.format(getUnitCost()));
		}
		if (getUnitPrice() != null) {
			setFormattedPrice(nf.format(getUnitPrice()));
		}
		if (getAltPrice() != null) {
			setFormattedAltPrice(nf.format(getAltPrice()));
		}
	}

	public Map buildSkuMap(Collection skus) {
		if (skus == null) {
			return null;
		}
		Map skuMap = new HashMap();
		Iterator i = skus.iterator();
		while (i.hasNext()) {
			SKU sku = (SKU) i.next();
			StringBuffer key = new StringBuffer();
			if (sku.getProductID() != null
					&& sku.getProductID().intValue() != 0) {
				key.append(sku.getProductID());
				key.append("|");
			}
			if (sku.getAttributeID1() != null
					&& sku.getAttributeID1().intValue() != 0) {
				key.append(sku.getAttributeID1());
				key.append("|");
				if (sku.getOptionID1() != null
						&& sku.getOptionID1().intValue() != 0) {
					key.append(sku.getOptionID1());
					key.append("|");
				} else {
					key.append("0|");
				}
			}
			if (sku.getAttributeID2() != null
					&& sku.getAttributeID2().intValue() != 0) {
				key.append(sku.getAttributeID2());
				key.append("|");
				if (sku.getOptionID2() != null
						&& sku.getOptionID2().intValue() != 0) {
					key.append(sku.getOptionID2());
					key.append("|");
				} else {
					key.append("0|");
				}
			}
			if (sku.getAttributeID3() != null
					&& sku.getAttributeID3().intValue() != 0) {
				key.append(sku.getAttributeID3());
				key.append("|");
				if (sku.getOptionID3() != null
						&& sku.getOptionID3().intValue() != 0) {
					key.append(sku.getOptionID3());
					key.append("|");
				} else {
					key.append("0|");
				}
			}
			if (sku.getAttributeID4() != null
					&& sku.getAttributeID4().intValue() != 0) {
				key.append(sku.getAttributeID4());
				key.append("|");
				if (sku.getOptionID4() != null
						&& sku.getOptionID4().intValue() != 0) {
					key.append(sku.getOptionID4());
					key.append("|");
				} else {
					key.append("0|");
				}
			}
			if (sku.getAttributeID5() != null
					&& sku.getAttributeID5().intValue() != 0) {
				key.append(sku.getAttributeID5());
				key.append("|");
				if (sku.getOptionID5() != null
						&& sku.getOptionID5().intValue() != 0) {
					key.append(sku.getOptionID5());
					key.append("|");
				} else {
					key.append("0|");
				}
			}
			if (log.isDebugEnabled())
				log.debug("key: " + key);
			skuMap.put(key.toString(), sku);
		}
		return skuMap;
	}
}
