package com.softslate.commerce.customer.customer;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.AddressBookAddressForm;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class AddAddressAction extends BaseAction {
	static Log log = LogFactory.getLog(AddAddressAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		AddressBookAddressForm baseForm = (AddressBookAddressForm) form;
		synchronized (AppUtils.getSessionLock(request.getSession())) {
			request.getSession().setAttribute("shippingAndTaxesProcessed",
					"false");

			// Preserve the original request so the customer is redirected
			// back to the main addresses page after logging in
			request.setAttribute("originalRequest", "/AccountAddresses.do");

			// If the user is not logged in, it's an error.
			if (baseForm.getUser() == null
					|| baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			String allowAddressBook = (String) baseForm.getSettings().getValue(
					"allowAddressBook");
			String maxCustomerAddressBook = (String) baseForm.getSettings()
					.getValue("maxCustomerAddressBook");
			if (!"1".equals(allowAddressBook)) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.addressBookNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Error: " + baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			int maxCustomerAddrBk = 1;
			try {
				maxCustomerAddrBk = Integer.parseInt(maxCustomerAddressBook);
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log
							.error("Invalid maximum customer address book number, using a default of 1!");
				}
			}
			int currentAddresses = 0;
			Collection addresses = baseForm.getUser().getCustomer()
					.getCustomerAddressCollection();
			if (addresses != null && addresses.size() > 0) {
				currentAddresses = addresses.size();
			}
			if (currentAddresses >= maxCustomerAddrBk) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.addressBookMax"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			CustomerAddressProcessor customerAddressProcessor = (CustomerAddressProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerAddressProcessorImplementer");

			// Grab the specific fields we need as the form may be billing or
			// delivery
			Map addressParams;
			if (baseForm.getIsBilling()) {
				addressParams = baseForm.describeAddressType("billing");
			} else {
				addressParams = baseForm.describeAddressType("delivery");
			}
			if (addressParams != null) {
				// Add in the non-address fields for the update
				addressParams.put("useForBilling", baseForm.getUseForBilling());
				addressParams.put("isBilling", baseForm.getIsBilling());
			}

			Map result = customerAddressProcessor.addAddressBookAddress(addressParams);

			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				String resultCode = (String) result.get("resultCode");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage(resultCode));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Add a message to tell the user it worked
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.addressBookAdded"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			return mapping.findForward("success");
		}
	}
}
