package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Action to handle customer password reset
 * 
 * @author Mike Groves, jmcwill
 */
public class CompleteResetPasswordAction extends BaseAction {

	static Log log = LogFactory.getLog(CompleteResetPasswordAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		if (log.isInfoEnabled())
			log.info("Starting action.");
		CompleteResetPasswordForm baseForm = (CompleteResetPasswordForm) form;

		String useAdvancedLostPassword = (String) baseForm.getSettings().getValue("useAdvancedLostPassword");
		if (!useAdvancedLostPassword.equals("1")) {
			response.setStatus(HttpServletResponse.SC_NOT_FOUND);
			return mapping.findForward("failureOff");
		}

		CustomerProcessor customerProcessor = (CustomerProcessor) baseForm.getBusinessObjectFactory().createObject(
				"customerProcessorImplementer");
		Map result = customerProcessor.resetPassword(baseForm.getPasswordToken(), baseForm.getDecryptedPassword());

		if (result != null && result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
			String resultCode = (String) result.get("resultCode");
			if (resultCode.equals("errors.sameAsPreviousPassword")) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(resultCode,
						baseForm.getSettings().getValue("numberOfPreviousPasswordsToCheckCustomer")));
			} else {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(resultCode));
			}
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): " + baseForm.getErrors().toString());
			return mapping.findForward("failure");
		}

		Customer customer = (Customer) result.get("customer");

		String validEmail = null;
		if (baseForm.formUtils().isValidEmail(customer.getUserName())) {
			validEmail = customer.getUserName();
		} else if (baseForm.formUtils().isValidEmail(customer.getEmail1())) {
			validEmail = customer.getEmail1();
		} else {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.validEmailNotFound"));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			return mapping.findForward("failure");
		}

		if (StringUtils.isBlank(baseForm.getSettings().getValue("passwordResetEmailSubject"))) {

			// Use a Template (newest implementation)
			customerProcessor.sendCompleteResetPasswordEmail(customer);
		} else {
			try {

				// If email settings are defined, use those (older implementation)
				ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory()
						.createUtilsObject("actionUtilsImplementer");
				String fromEmail = (String) baseForm.getSettings().getValue("storeEmailAddress");
				String smtpServer = (String) baseForm.getSettings().getValue("storeSMTPServer");
				String smtpUsername = (String) baseForm.getSettings().getValue("storeSMTPUsername");
				String smtpPassword = (String) baseForm.getSettings().getValue("storeSMTPPassword");
				String smtpPort = (String) baseForm.getSettings().getValue("storeSMTPPort");
				String characterEncoding = (String) baseForm.getSettings().getValue("characterEncoding");
				String subject = (String) baseForm.getSettings().getValue("passwordResetEmailSubject");
				String textMessage = (String) baseForm.getSettings().getValue("passwordResetEmailTextTemplate");
				String htmlMessage = (String) baseForm.getSettings().getValue("passwordResetEmailHTMLTemplate");
				subject = actionUtils.parseLostPasswordTemplateAdvanced(customer, baseForm, subject, "");
				textMessage = actionUtils.parseLostPasswordTemplateAdvanced(customer, baseForm, textMessage, "");
				htmlMessage = actionUtils.parseLostPasswordTemplateAdvanced(customer, baseForm, htmlMessage, "");
				actionUtils.sendMultipartEmail(validEmail, "", "", fromEmail.trim(), smtpServer, subject, textMessage,
						htmlMessage, smtpUsername.trim(), smtpPassword.trim(), smtpPort.trim(),
						characterEncoding.trim());
			} catch (Exception e) {
				if (log.isErrorEnabled())
					log.error(e.toString(), e);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.sendingEmail"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		}

		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.passwordWasReset"));
		saveMessages(baseForm.getRequest(), baseForm.getMessages());

		// We want to forward to login screen, but you can't clear the baseForm
		// of the password because it will use loginForm bean and re-pop it. Set
		// a request var to let the system know not to show the form that way.
		baseForm.getRequest().setAttribute("resetPasswordSuccess", "true");

		return mapping.findForward("success");
	}
}
