package com.softslate.commerce.customer.customer;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions to add or edit a wish list. Used for
 * requests to "/WishListAddForm.do", "/WishListAdd.do", "WishListEditForm.do",
 * "WishListEdit.do"
 * 
 * @author Jason McWilliams
 * 
 */
public class WishListAddEditForm extends BaseForm {

	private static final long serialVersionUID = 3001929450641723798L;

	static Log log = LogFactory.getLog(WishListAddEditForm.class);

	private String customerWishListID = null;

	private String name = null;

	private String description = null;

	public String getCustomerWishListID() {
		return customerWishListID;
	}

	public void setCustomerWishListID(String customerWishListID) {
		this.customerWishListID = customerWishListID;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		if (getCustomerWishListID() != null
				&& getCustomerWishListID().length() > 0) {
			try {
				Integer.parseInt(getCustomerWishListID());
			} catch (NumberFormatException nfe) {
				String message = getMessage("core", "errors.wishListNotFound",
						null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}
		}

		// Name is required
		if (getName() == null || getName().trim().length() < 1) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Name"));
		}

		// Max length on the name field
		if (getName() != null && getName().trim().length() > 255) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.maxlength", "Name", "255"));
		}

		// Max length on the description field
		if (getDescription() != null && getDescription().trim().length() > 255) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.maxlength", "Description", "255"));
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}
}
