package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions from the combo form during checkout. Used
 * for requests to "/CheckoutCombo.do".
 * <p>
 * The <code>execute</code> method of this class first clears the session's
 * "shippingAndTaxesProcessed" attribute in case a new shipping method has been
 * selected. It then runs
 * {@link com.softslate.commerce.customer.core.ActionUtils#findCheckoutScreen(BaseForm)}
 * to verifiy that the user has made it this far in the checkout process.
 * Finally, it invokes
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processCheckoutAddresses(BaseForm)}
 * to process the checkout addresses, shipping, and taxes, followed by
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processCheckoutPayment(BaseForm)}
 * to process the payment information.
 * 
 * @author David Tobey
 */
public class CheckoutComboAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CheckoutComboAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		BaseForm baseForm = (BaseForm) form;

		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {
			// If user is resubmitting, clear the session of previous payment
			// info.
			baseForm.getRequest().getSession().setAttribute(
					"validatedPaymentForm", null);
			baseForm.getRequest().getSession().setAttribute("paymentProcessor",
					null);
			// Make sure the user is allowed to have this action performed.
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String checkoutScreen = actionUtils.findCheckoutScreen(baseForm);
			if (!checkoutScreen.equals("checkoutCombo")) {
				return baseForm.getMapping().findForward(checkoutScreen);
			}
			// Process the addresses.
			ActionForward af = processCheckoutAddresses(baseForm, response);
			// Return if failure or if no shipping is required.
			String shippingRequired = (String) baseForm.getSettings().getValue(
					"shippingRequired");
			String paymentRequired = (String) baseForm.getSettings().getValue(
					"paymentRequired");
			if (!af.getName().startsWith("success")
					|| (shippingRequired.equals("0") && (paymentRequired
							.equals("0") || (baseForm.getUser().getOrder()
							.getSubtotal().doubleValue() == 0 && baseForm
							.getUser().getOrder().getTotal().doubleValue() == 0)))) {
				return af;
			}
			// Process the shipping.
			if (shippingRequired.equals("1")) {
				af = processCheckoutShipping(baseForm);
			}
			// Return if failure or if no payment is required.
			if (!af.getName().startsWith("success")
					|| paymentRequired.equals("0")
					|| (baseForm.getUser().getOrder().getSubtotal()
							.doubleValue() == 0 && baseForm.getUser()
							.getOrder().getTotal().doubleValue() == 0)) {
				return af;
			}
			// Process the payment.
			return processCheckoutPayment(baseForm);
		}
	}
}
