package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions of the checkout addresses form. Used for
 * requests to "/CheckoutComboShipping.do".
 * <p>
 * The <code>execute</code> method of this class first clears the session's
 * "shippingAndTaxesProcessed" attribute in case a new shipping method has been
 * selected. It then runs
 * {@link com.softslate.commerce.customer.core.ActionUtils#findCheckoutScreen(BaseForm)}
 * to verifiy that the user has made it this far in the checkout process.
 * Finally, it invokes
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processCheckoutAddresses(BaseForm)}
 * followed by
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processCheckoutShipping(BaseForm)}
 * to process the checkout addresses, shipping, and taxes.
 * 
 * @author David Tobey
 */
public class CheckoutComboShippingAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CheckoutComboShippingAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		BaseForm baseForm = (BaseForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {
			// Make sure the user is allowed to have this action performed.
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String checkoutScreen = actionUtils.findCheckoutScreen(baseForm);
			if (!checkoutScreen.equals("checkoutComboShipping")) {
				return baseForm.getMapping().findForward(checkoutScreen);
			}
			if (log.isDebugEnabled())
				log.debug("paymentRequired: "
						+ baseForm.getSettings().getValue(
				"paymentRequired"));
			// Process the addresses.
			ActionForward af = processCheckoutAddresses(baseForm, response);
			// Return if failure or if no shipping is required.
			String shippingRequired = (String) baseForm.getSettings().getValue(
					"shippingRequired");
			if (!af.getName().startsWith("success")
					|| shippingRequired.equals("0")) {
				return af;
			}
			return processCheckoutShipping(baseForm);
		}
	}
}
