package com.softslate.commerce.customer.payment;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.payment.AuthorizeNetAPIProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;

import net.authorize.api.contract.v1.CustomerPaymentProfileMaskedType;

/**
 * Struts form class for submissions of payment information for Authorize.net
 * during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.AuthorizeNetProcessor}
 * payment processor.
 * <p>
 * The database settings "activePaymentProcessors", "activePaymentForms", and
 * "activePaymentFormTemplate" represent the various payment methods available
 * to users. During checkout, each of the "activePaymentForms" will be
 * instantiated and prepared for display on the checkout payment screen. When
 * the user submits payment information the <code>validate</code> method of
 * the corresponding payment form is invoked.
 * <p>
 * The <code>validate</code> method of this class ensures that the incoming
 * credit card information is valid and complete. It then parses the database
 * settings for various PayflowLink fields and populates the corresponding form
 * properties.
 * 
 * @author David Tobey
 */
public class AuthorizeNetForm extends BasicPaymentForm {

	private static final long serialVersionUID = 2846001494308159452L;

	static Log log = LogFactory.getLog(AuthorizeNetForm.class);

	private String description = null;

	private String customerID = null;

	private String invoiceNumber = null;

	private String poNumber = null;

	private String ipAddress = null;

	private String acceptJsDataValue = null;

	private String acceptJsDataDesc = null;

	private String acceptJsCheckTotal = null;

	private String savePaymentInformation = null;

	private String paymentProfileID = null;

	// Used when the embedded iframe returns a response
	private String action = null;

	private String response = null;

	public String getCustomerID() {
		return customerID;
	}

	public void setCustomerID(String customerID) {
		this.customerID = customerID;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	public String getPoNumber() {
		return poNumber;
	}

	public void setPoNumber(String poNumber) {
		this.poNumber = poNumber;
	}

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public String getAcceptJsDataValue() {
		return acceptJsDataValue;
	}

	public void setAcceptJsDataValue(String acceptJsDataValue) {
		this.acceptJsDataValue = acceptJsDataValue;
	}

	public String getAcceptJsDataDesc() {
		return acceptJsDataDesc;
	}

	public void setAcceptJsDataDesc(String acceptJsDataDesc) {
		this.acceptJsDataDesc = acceptJsDataDesc;
	}

	public String getAcceptJsCheckTotal() {
		return acceptJsCheckTotal;
	}

	public void setAcceptJsCheckTotal(String acceptJsCheckTotal) {
		this.acceptJsCheckTotal = acceptJsCheckTotal;
	}

	public String getSavePaymentInformation() {
		return savePaymentInformation;
	}

	public void setSavePaymentInformation(String savePaymentInformation) {
		this.savePaymentInformation = savePaymentInformation;
	}

	public String getPaymentProfileID() {
		return paymentProfileID;
	}

	public void setPaymentProfileID(String paymentProfileID) {
		this.paymentProfileID = paymentProfileID;
	}

	public String getAction() {
		return action;
	}

	public void setAction(String action) {
		this.action = action;
	}

	public String getResponse() {
		return response;
	}

	public void setResponse(String response) {
		this.response = response;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		// Skip if the customer is paying with a profile or a response from an embedded iframe is being relayed
		if (StringUtils.isBlank(getPaymentProfileID()) && (StringUtils.isBlank(getAction()))) {
			validateName();
			if (validateCardAndCode()) {
				validateDate();
			}
			validateAcceptJsParams();
			if (validateCardAndCode()) {
				validateSecurityCode();
			}
			validateType();
			if (validateCardAndCode()) {
				validateNumber();
			}
		}
		validateJcaptcha();
		ActionUtils actionUtils = (ActionUtils) getBusinessObjectFactory().createUtilsObject("actionUtilsImplementer");
		populateReferenceFields(actionUtils);
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

	protected void populateReferenceFields(ActionUtils actionUtils) {

		// Populate description, comments, etc., parsing them for variables.
		try {
			setDescription(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetDescription")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setInvoiceNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetInvoiceNum")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setPoNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetPONumber")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setCustomerID(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetCustomerID")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setIpAddress(getRequest().getRemoteAddr());
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		if (log.isDebugEnabled()) {
			log.debug("description: " + getDescription());
			log.debug("invoice: " + getInvoiceNumber());
			log.debug("ponum: " + getPoNumber());
			log.debug("customerID: " + getCustomerID());
			log.debug("ipAddress: " + getIpAddress());
		}

	}

	protected void validateAcceptJsParams() {
		if ("AcceptJS".equals(getSettings().getValue("authorizeNetIntegrationMethod"))
				&& !("1".equals(getSettings().getValue("authorizeNetEnableCustomerProfiles"))
						&& StringUtils.isNotBlank(getPaymentProfileID()))) {
			if (StringUtils.isEmpty(getAcceptJsDataValue()) || StringUtils.isEmpty(getAcceptJsDataDesc())) {
				String message = getMessage("payment", "payment.errorAuthorizeNetAPIProcessorNoNonce", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
			}
			if (StringUtils.isEmpty(getAcceptJsCheckTotal())
					|| !(new Double(getAcceptJsCheckTotal())).equals(getUser().getOrder().getTotal())) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalid", "Submitted total"));
			}
		}
	}

	protected boolean validateCardAndCode() {
		return !"AcceptJS".equals(getSettings().getValue("authorizeNetIntegrationMethod"));
	}

	public void preparePaymentFormHook(Boolean validating, BaseForm baseForm) throws Exception {

		if (!validating) {

			AuthorizeNetAPIProcessor apiProcessor = (AuthorizeNetAPIProcessor) baseForm.getBusinessObjectFactory()
					.createObject("authorizeNetAPIProcessorImplementer");
			User user = (User) baseForm.getRequest().getSession().getAttribute("user");
			Settings settings = (Settings) baseForm.getRequest().getAttribute("settings");
			setUser(user);
			setSettings(settings);
			setBusinessObjectFactory(baseForm.getBusinessObjectFactory());

			// Customer profile integration.
			// For AcceptHosted, if we want the 'save payment option' to appear on their form we have to have a customer
			// profile. So create one now for the customer if one doesn't exist.
			if (settings.getValue("authorizeNetIntegrationMethod").startsWith("AcceptHosted")
					&& "1".equals(settings.getValue("authorizeNetEnableCustomerProfiles"))
					&& "1".equals(settings.getValue("authorizeNetHostedAddPaymentProfile"))
					&& user.getCustomer() != null && StringUtils.isBlank(user.getCustomer().getCimProfileID())) {
				try {
					Map results = apiProcessor.createCustomerProfile(user.getCustomer(), null);
					synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {
						user.getCustomer().setCimProfileID((String) results.get("newProfileID"));
					}
				} catch (Exception e) {
					log.error("Exception creating a new customer profile (swallowing): ", e);
				}
			}

			// Customer profile integration.
			// If we aren't using AcceptHosted and we are displaying the form ourselves, retrieve any existing payment
			// profiles from Authorize.net and put them in the request scope for our form to display.
			else if (!settings.getValue("authorizeNetIntegrationMethod").startsWith("AcceptHosted")
					&& "1".equals(settings.getValue("authorizeNetEnableCustomerProfiles")) && user.getCustomer() != null
					&& StringUtils.isNotBlank(user.getCustomer().getCimProfileID())) {
				try {
					List<CustomerPaymentProfileMaskedType> paymentProfiles = apiProcessor
							.retrievePaymentProfiles(user.getCustomer().getCimProfileID(), false);
					baseForm.getRequest().setAttribute("paymentProfiles", paymentProfiles);
				} catch (Exception e) {
					log.error("Error retrieving existing payment profiles for customer (swallowing): ", e);
				}
			}

			// Accept Hosted integration. Retrieve a hosted payment page token and signal the JSP to render it in place
			// of the normal payment form.
			if (settings.getValue("authorizeNetIntegrationMethod").startsWith("AcceptHosted")) {
				String hostedPageToken = null;
				String errorCode = null;
				try {
					ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory()
							.createUtilsObject("actionUtilsImplementer");
					populateReferenceFields(actionUtils);
					Map parameters = new HashMap();
					parameters.put("description", getDescription());
					parameters.put("invoiceNumber", getInvoiceNumber());
					parameters.put("poNumber", getPoNumber());
					parameters.put("customerID", getCustomerID());
					parameters.put("ipAddress", getIpAddress());
					Map hostedPaymentPageResult = apiProcessor.hostedPaymentPage(parameters);
					hostedPageToken = (String) hostedPaymentPageResult.get("hostedPageToken");
				} catch (Exception e) {
					log.error("Exception getting a hosted token: ", e);
					errorCode = "errors.acceptHostedCommunicationError";
				}

				// Make sure it was created
				if (StringUtils.isBlank(hostedPageToken)) {
					log.error("Error getting a hosted token");
					errorCode = "errors.acceptHostedCommunicationError";
				}
				if (errorCode != null) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(errorCode));
					baseForm.getRequest().setAttribute(Globals.ERROR_KEY, baseForm.getErrors());
					if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
						log.info("Validation error(s): " + baseForm.getErrors().toString());
				}
				baseForm.getRequest().setAttribute("hostedPageToken", hostedPageToken);
			}
		}
	}
}
