package com.softslate.commerce.customer.payment;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.ActionUtils;

/**
 * Struts form class for submissions of payment information for PayPal Payflow
 * Pro during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.PayflowProProcessor}
 * payment processor.
 * <p>
 * The database settings "activePaymentProcessors", "activePaymentForms", and
 * "activePaymentFormTemplate" represent the various payment methods available
 * to users. During checkout, each of the "activePaymentForms" will be
 * instantiated and prepared for display on the checkout payment screen. When
 * the user submits payment information the <code>validate</code> method of
 * the corresponding payment form is invoked.
 * <p>
 * The <code>validate</code> method of this class ensures that the incoming
 * credit card information is valid and complete. It then parses the database
 * settings for various Payflow Pro fields and populates the corresponding form
 * properties. It also sets the cert path for Payflow Pro equal to the location
 * of /WEB-INF/conf/keys on the server.
 * 
 * @author David Tobey
 */
public class PayflowProForm extends BasicPaymentForm {

	private static final long serialVersionUID = 3257846563064918321L;

	static Log log = LogFactory.getLog(PayflowProForm.class);

	private String comment1 = null;

	private String comment2 = null;

	private String certPath = null;

	private String custRef = null;

	private String customerCode = null;

	private String poNumber = null;

	public String getComment1() {
		return comment1;
	}

	public void setComment1(String comment1) {
		this.comment1 = comment1;
	}

	public String getComment2() {
		return comment2;
	}

	public void setComment2(String comment2) {
		this.comment2 = comment2;
	}

	public String getCertPath() {
		return certPath;
	}

	public void setCertPath(String certPath) {
		this.certPath = certPath;
	}

	public String getCustRef() {
		return custRef;
	}

	public void setCustRef(String custRef) {
		this.custRef = custRef;
	}

	public String getCustomerCode() {
		return customerCode;
	}

	public void setCustomerCode(String customerCode) {
		this.customerCode = customerCode;
	}

	public String getPoNumber() {
		return poNumber;
	}

	public void setPoNumber(String poNumber) {
		this.poNumber = poNumber;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		super.validate(mapping, request);
		return payflowValidate();
	}

	public ActionErrors payflowValidate() {
		ActionUtils actionUtils = (ActionUtils) getBusinessObjectFactory()
				.createUtilsObject("actionUtilsImplementer");
		// Set the cert path
		setCertPath(getServlet().getServletContext().getRealPath(
				"/WEB-INF/conf/keys"));

		if (log.isDebugEnabled())
			log.debug("Cert path: " + getCertPath());
		// Populate comments, etc., parsing them for variables.
		try {
			setComment1(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowProComment1")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setComment2(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowProComment2")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setCustRef(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowProCustRef")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setCustomerCode(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowProCustomerCode")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setPoNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowProPONumber")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		if (log.isDebugEnabled()) {
			log.debug("comment1: " + getComment1());
			log.debug("comment2: " + getComment2());
			log.debug("invoice: " + getCustomerCode());
			log.debug("ponum: " + getPoNumber());
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

}
