package com.softslate.commerce.daos.email;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.email.Email;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to
 * Emails.
 */
public class EmailGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements EmailGatewayDAO {

	static Log log = LogFactory.getLog(EmailGatewayDAOHibernate.class);

	public Map loadEmailsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public void editDeleteEmails(Map parameters, BusinessObjectFactory bof) throws Exception {
		editDeleteItems(parameters, bof);
	}

	public Collection<Email> loadAllEmails() throws DataAccessException {
		int maxRecords = 1000;
		String maxAdminRecordsToDisplay = (String) getSettings().getValue("maxAdminRecordsToDisplay");
		if (maxAdminRecordsToDisplay != null) {
			maxRecords = Integer.parseInt(maxAdminRecordsToDisplay);
		}
		Collection<Email> emails = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "email.selectAll");
			q.setMaxResults(maxRecords);
			emails = new LinkedHashSet<Email>(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return emails;
	}

	public Integer emailsCountByTypeAndOrderID(String type, int orderID) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "email.emailsCountByTypeAndOrderID");
			q.setString("type", type);
			q.setInteger("orderID", orderID);
			return new Integer(((Long) q.uniqueResult()).intValue());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Integer emailsCountByTypeAndCustomerID(String type, int customerID) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "email.emailsCountByTypeAndCustomerID");
			q.setString("type", type);
			q.setInteger("customerID", customerID);
			return new Integer(((Long) q.uniqueResult()).intValue());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Integer emailsCountByTypeAndEmailAddress(String type, String emailAddress) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "email.emailsCountByTypeAndEmailAddress");
			q.setString("type", type);
			q.setString("emailAddress", emailAddress);
			return new Integer(((Long) q.uniqueResult()).intValue());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Integer emailsCountByTypeAndEmailAddressAfterCreatedTime(String type, String emailAddress, String createdTime)
			throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "email.emailsCountByTypeAndEmailAddressAfterCreatedTime");
			q.setString("type", type);
			q.setString("emailAddress", emailAddress);
			q.setString("createdTime", createdTime);
			return new Integer(((Long) q.uniqueResult()).intValue());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

}
