package com.softslate.commerce.daos.order;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.State;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscState</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class StateDAOHibernate extends BaseDAO implements DAOInterface,
        StateDAO {

    static Log log = LogFactory.getLog(StateDAOHibernate.class);

    private State state = null;

    public State getState() {
        return (this.state);
    }

    public void setState(State state) {
        this.state = state;
    }

    public void loadStateFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            State record = (State) session.get(State.class, new Integer(
                    getState().getStateID()));
            PropertyUtils.copyProperties(getState(), record);
            setState(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

	public void loadStateFromCode() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "state.selectStateFromCode");
			q.setString("code", this.getState().getCode());
			State loadedState = (State) q.uniqueResult();
			setState(loadedState);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

    public void insertState() throws DataAccessException {
        insertState(false);
    }

    public void insertState(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getState());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateState() throws DataAccessException {
        updateState(false);
    }

    public void updateState(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getState());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteState() throws Exception {
        deleteState(false);
    }

    public void deleteState(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            setState((State) session.load(State.class, new Integer(getState()
                    .getStateID())));
            session.delete(getState());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

	public Object loadObject(Object object) throws Exception {
		if (null != object && object instanceof State) {
			this.setState((State) object);
			if (0 != this.getState().getStateID()) {
				this.loadStateFromID();
				return this.getState();
			} else if (null != this.getState().getCode()) {
				this.loadStateFromCode();
				return this.getState();
			}
		}
		return null;
	}
}
