package com.softslate.commerce.daos.product;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscProductSetting</code> table of the
 * database.
 * 
 * @author David Tobey
 */
public class ProductSettingDAOHibernate extends BaseDAO implements DAOInterface, ProductSettingDAO {

	static Log log = LogFactory.getLog(ProductSettingDAOHibernate.class);

	private ProductSetting productSetting = null;

	public ProductSetting getProductSetting() {
		return (this.productSetting);
	}

	public void setProductSetting(ProductSetting productSetting) {
		this.productSetting = productSetting;
	}

	public void loadProductSettingFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			ProductSetting item = (ProductSetting) session.get(ProductSetting.class, new Integer(getProductSetting()
					.getProductSettingID()));
			PropertyUtils.copyProperties(getProductSetting(), item);
			setProductSetting(item);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadProductSettingFromCode() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "productSetting.selectFromCodeAndProductID");
			q.setString("code", getProductSetting().getCode());
			q.setInteger("productID", getProductSetting().getProductID());
			q.setCacheable(true);
			ProductSetting item = (ProductSetting) q.uniqueResult();
			setProductSetting(item);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertProductSetting() throws DataAccessException {
		insertProductSetting(false);
	}

	public void insertProductSetting(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(getProductSetting());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateProductSetting() throws DataAccessException {
		updateProductSetting(false);
	}

	public void updateProductSetting(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getProductSetting());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteProductSetting() throws DataAccessException {
		deleteProductSetting(false);
	}

	public void deleteProductSetting(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setProductSetting((ProductSetting) session.load(ProductSetting.class,
					new Integer(getProductSetting().getProductSettingID())));
			session.delete(getProductSetting());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Object loadObject(Object object) throws Exception {
		if (null != object && object instanceof ProductSetting) {
			this.setProductSetting((ProductSetting) object);
			if (0 != this.getProductSetting().getProductSettingID()) {
				this.loadProductSettingFromID();
			} else if (null != this.getProductSetting().getCode() && 0 != this.getProductSetting().getProductID()) {
				this.loadProductSettingFromCode();
			}
			return this.getProductSetting();
		}
		return null;
	}

	public boolean deleteObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof ProductSetting) {
			this.setProductSetting((ProductSetting) object);
			this.deleteProductSetting(commit);
			return true;
		}
		return false;
	}

	public Object insertObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof ProductSetting) {
			this.setProductSetting((ProductSetting) object);
			this.insertProductSetting(commit);
			return this.getProductSetting();
		}
		return null;
	}

	public Object updateObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof ProductSetting) {
			this.setProductSetting((ProductSetting) object);
			this.updateProductSetting(commit);
			return this.getProductSetting();
		}
		return null;

	}
}
