package com.softslate.commerce.daos.tax;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to basic tax rates.
 * 
 * @author David Tobey
 */
public class BasicTaxRateGatewayDAOHibernate extends
		BaseAdminGatewayDAOHibernate implements BasicTaxRateGatewayDAO {

	static Log log = LogFactory.getLog(BasicTaxRateGatewayDAOHibernate.class);

	public Map loadBasicTaxRatesAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteTaxRates(Map parameters, BusinessObjectFactory bof)
			throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public Collection loadActiveBasicTaxRates() throws Exception {
        if (log.isInfoEnabled())
            log.info("Loading active tax rates.");
		Collection rates = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "basicTaxRate.selectActive");
			rates = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return rates;
	}

	public int loadMaxBasicTaxRateID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "basicTaxRate.maxBasicTaxRateID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllBasicTaxRatesInRange(int firstID, int lastID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading BasicTaxRates within an id range: " + firstID + "-" + lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "basicTaxRate.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}
}
